! string_buffer.f90 - string buffer management
! vi: set sw=4:
! Copyright (C) TOYODA Eizi, 2000.  All rights reserved.
!
! 2000-01-12 Lcpi	 (netcdf_filename.f90)
! 2000-01-16 Lcpi	X^CnbN
! 2000-02-16 Lcpi	ϒ񃉃Cû߂ɊD

module string_buffer

    implicit none
    private

    integer, parameter:: INITIAL_TABLE_SIZE = 4

    type TableEntry
    	integer::			nlink
	character, pointer::		name(:)
    end type

    type(TableEntry), save, pointer::	Table(:)

contains

    integer function lookupId(id)
	integer, intent(in)::		id
	integer::			i
    continue
	do, i = 1, size(Table)
	    if (Table(i)%id == id) then
		lookupId = i
		return
	    endif
	enddo
	lookupId = -1
    end function

    function initTable()
	logical::		initTable
	integer::		status
    continue
	initTable = .true.
	if (associated(Table)) return
	allocate(Table(INITIAL_TABLE_SIZE), stat=status)
	if (status == 0) then
	    Table%id = -1
	else
	    initTable = .false.
	endif
    end function

    subroutine growTable(newsize)
	integer, intent(in)::		newsize
	integer::			copysize, status
	type(TableEntry), pointer::	newtable(:)
    continue
	copysize = min(size(Table), newsize)
	allocate(newtable(newsize), stat=status)
	if (status == 0) then
	    newtable(1:copysize) = Table(1:copysize)
	    newtable(copysize+1:)%id = -1
	    deallocate(Table)
	    Table => newtable
	endif
    end subroutine

    subroutine StringDelete(id)
	integer, intent(in)::		id
	integer::			i
    continue
	i = lookupId(id)
	Table(i)%id = -1
    end subroutine

    function StringIdToString(id) result(result)
	character(len = size(Table(id)%name))::		result
	integer, intent(in)::		id
	integer::			i
    continue
	i = lookupId(id)
	result = Table(i)%name
    end function

    subroutine StringSaveChars(id, name)
	integer, intent(in)::		id
	character(len=*), intent(in)::	name
	logical::			initTable
	integer::			i
    continue
	if (.not. initTable()) return
	i = lookupId(id)
	if (i /= -1) then
	    Table(i)%name = name
	    return
	endif
	i = lookupId(-1)
	if (i /= -1) then
	    Table(i)%id = id
	    Table(i)%name = name
	    return
	endif
	call growTable(size(Table) * 2)
	i = lookupId(-1)
	if (i /= -1) then
	    Table(i)%id = id
	    Table(i)%name = name
	endif
    end subroutine

end module
